package com.sun.org.apache.xml.internal.security.signature;

import com.sun.org.slf4j.internal.LoggerFactory;
import java.security.AccessController;
import com.sun.org.apache.xml.internal.security.utils.UnsyncBufferedOutputStream;
import com.sun.org.apache.xml.internal.security.utils.DigesterOutputStream;
import java.util.Iterator;
import java.util.Set;
import com.sun.org.apache.xml.internal.security.signature.reference.ReferenceNodeSetData;
import com.sun.org.apache.xml.internal.security.signature.reference.ReferenceSubTreeData;
import com.sun.org.apache.xml.internal.security.signature.reference.ReferenceOctetStreamData;
import com.sun.org.apache.xml.internal.security.transforms.InvalidTransformException;
import com.sun.org.apache.xml.internal.security.transforms.params.InclusiveNamespaces;
import java.util.HashSet;
import com.sun.org.apache.xml.internal.security.transforms.Transform;
import java.io.IOException;
import com.sun.org.apache.xml.internal.security.transforms.TransformationException;
import com.sun.org.apache.xml.internal.security.c14n.InvalidCanonicalizerException;
import com.sun.org.apache.xml.internal.security.c14n.CanonicalizationException;
import java.io.OutputStream;
import org.w3c.dom.Attr;
import com.sun.org.apache.xml.internal.security.utils.resolver.ResourceResolverException;
import com.sun.org.apache.xml.internal.security.utils.resolver.ResourceResolver;
import org.w3c.dom.Node;
import com.sun.org.apache.xml.internal.security.algorithms.MessageDigestAlgorithm;
import com.sun.org.apache.xml.internal.security.exceptions.XMLSecurityException;
import com.sun.org.apache.xml.internal.security.utils.XMLUtils;
import com.sun.org.apache.xml.internal.security.algorithms.Algorithm;
import org.w3c.dom.Document;
import com.sun.org.apache.xml.internal.security.signature.reference.ReferenceData;
import org.w3c.dom.Element;
import com.sun.org.apache.xml.internal.security.transforms.Transforms;
import com.sun.org.slf4j.internal.Logger;
import com.sun.org.apache.xml.internal.security.utils.SignatureElementProxy;

public class Reference extends SignatureElementProxy
{
    public static final String OBJECT_URI = "http://www.w3.org/2000/09/xmldsig#Object";
    public static final String MANIFEST_URI = "http://www.w3.org/2000/09/xmldsig#Manifest";
    public static final int MAXIMUM_TRANSFORM_COUNT = 5;
    private boolean secureValidation;
    private static boolean useC14N11;
    private static final Logger LOG;
    private Manifest manifest;
    private XMLSignatureInput transformsOutput;
    private Transforms transforms;
    private Element digestMethodElem;
    private Element digestValueElement;
    private ReferenceData referenceData;
    
    protected Reference(Document doc, String baseURI, String uri, Manifest manifest, Transforms transforms, final String s) throws XMLSignatureException {
        super(doc);
        this.addReturnToSelf();
        this.baseURI = baseURI;
        this.manifest = manifest;
        this.setURI(uri);
        if (transforms != null) {
            this.appendSelf(this.transforms = transforms);
            this.addReturnToSelf();
        }
        this.appendSelf(this.digestMethodElem = new Algorithm(this.getDocument(), s) {
            @Override
            public String getBaseNamespace() {
                return "http://www.w3.org/2000/09/xmldsig#";
            }
            
            @Override
            public String getBaseLocalName() {
                return "DigestMethod";
            }
        }.getElement());
        this.addReturnToSelf();
        this.appendSelf(this.digestValueElement = XMLUtils.createElementInSignatureSpace(this.getDocument(), "DigestValue"));
        this.addReturnToSelf();
    }
    
    protected Reference(Element element, String s, Manifest manifest) throws XMLSecurityException {
        this(element, s, manifest, true);
    }
    
    protected Reference(Element element, String s, Manifest manifest, boolean b) throws XMLSecurityException {
        super(element, s);
        this.secureValidation = b;
        this.baseURI = s;
        Element element2 = XMLUtils.getNextElement(element.getFirstChild());
        if (element2 != null && "Transforms".equals(element2.getLocalName()) && "http://www.w3.org/2000/09/xmldsig#".equals(element2.getNamespaceURI())) {
            (this.transforms = new Transforms(element2, this.baseURI)).setSecureValidation(b);
            if (b && this.transforms.getLength() > 5) {
                throw new XMLSecurityException("signature.tooManyTransforms", new Object[] { Integer.valueOf(this.transforms.getLength()), Integer.valueOf(5) });
            }
            element2 = XMLUtils.getNextElement(element2.getNextSibling());
        }
        this.digestMethodElem = element2;
        if (this.digestMethodElem == null) {
            throw new XMLSecurityException("signature.Reference.NoDigestMethod");
        }
        this.digestValueElement = XMLUtils.getNextElement(this.digestMethodElem.getNextSibling());
        if (this.digestValueElement == null) {
            throw new XMLSecurityException("signature.Reference.NoDigestValue");
        }
        this.manifest = manifest;
        return;
    }
    
    public MessageDigestAlgorithm getMessageDigestAlgorithm() throws XMLSignatureException {
        if (this.digestMethodElem == null) {
            return null;
        }
        String attributeNS = this.digestMethodElem.getAttributeNS(null, "Algorithm");
        if ("".equals(attributeNS)) {
            return null;
        }
        if (this.secureValidation && "http://www.w3.org/2001/04/xmldsig-more#md5".equals(attributeNS)) {
            throw new XMLSignatureException("signature.signatureAlgorithm", new Object[] { attributeNS });
        }
        return MessageDigestAlgorithm.getInstance(this.getDocument(), attributeNS);
    }
    
    public void setURI(String value) {
        if (value != null) {
            this.setLocalAttribute("URI", value);
        }
    }
    
    public String getURI() {
        return this.getLocalAttribute("URI");
    }
    
    public void setId(String value) {
        if (value != null) {
            this.setLocalIdAttribute("Id", value);
        }
    }
    
    public String getId() {
        return this.getLocalAttribute("Id");
    }
    
    public void setType(String value) {
        if (value != null) {
            this.setLocalAttribute("Type", value);
        }
    }
    
    public String getType() {
        return this.getLocalAttribute("Type");
    }
    
    public boolean typeIsReferenceToObject() {
        return "http://www.w3.org/2000/09/xmldsig#Object".equals(this.getType());
    }
    
    public boolean typeIsReferenceToManifest() {
        return "http://www.w3.org/2000/09/xmldsig#Manifest".equals(this.getType());
    }
    
    private void setDigestValueElement(byte[] bytes) {
        for (Node node = this.digestValueElement.getFirstChild(); node != null; node = node.getNextSibling()) {
            this.digestValueElement.removeChild(node);
        }
        this.digestValueElement.appendChild(this.createText(XMLUtils.encodeToString(bytes)));
    }
    
    public void generateDigestValue() throws XMLSignatureException, ReferenceNotInitializedException {
        this.setDigestValueElement(this.calculateDigest(false));
    }
    
    public XMLSignatureInput getContentsBeforeTransformation() throws ReferenceNotInitializedException {
        try {
            Attr attributeNodeNS = this.getElement().getAttributeNodeNS(null, "URI");
            ResourceResolver instance = ResourceResolver.getInstance(attributeNodeNS, this.baseURI, this.manifest.getPerManifestResolvers(), this.secureValidation);
            instance.addProperties(this.manifest.getResolverProperties());
            return instance.resolve(attributeNodeNS, this.baseURI, this.secureValidation);
        }
        catch (ResourceResolverException ex) {
            throw new ReferenceNotInitializedException(ex);
        }
    }
    
    private XMLSignatureInput getContentsAfterTransformation(XMLSignatureInput xmlSignatureInput, OutputStream os) throws XMLSignatureException {
        try {
            Transforms transforms = this.getTransforms();
            if (transforms != null) {
                XMLSignatureInput performTransforms = transforms.performTransforms(xmlSignatureInput, os);
                this.transformsOutput = performTransforms;
                return performTransforms;
            }
            return xmlSignatureInput;
        }
        catch (ResourceResolverException ex) {
            throw new XMLSignatureException(ex);
        }
        catch (CanonicalizationException ex2) {
            throw new XMLSignatureException(ex2);
        }
        catch (InvalidCanonicalizerException ex3) {
            throw new XMLSignatureException(ex3);
        }
        catch (TransformationException ex4) {
            throw new XMLSignatureException(ex4);
        }
        catch (XMLSecurityException ex5) {
            throw new XMLSignatureException(ex5);
        }
    }
    
    public XMLSignatureInput getContentsAfterTransformation() throws XMLSignatureException {
        XMLSignatureInput contentsBeforeTransformation = this.getContentsBeforeTransformation();
        this.cacheDereferencedElement(contentsBeforeTransformation);
        return this.getContentsAfterTransformation(contentsBeforeTransformation, null);
    }
    
    public XMLSignatureInput getNodesetBeforeFirstCanonicalization() throws XMLSignatureException {
        try {
            XMLSignatureInput xmlSignatureInput = this.getContentsBeforeTransformation();
            this.cacheDereferencedElement(xmlSignatureInput);
            Transforms transforms = this.getTransforms();
            if (transforms != null) {
                int i = 0;
                while (i < transforms.getLength()) {
                    Transform item = transforms.item(i);
                    String uri = item.getURI();
                    if (!uri.equals("http://www.w3.org/2001/10/xml-exc-c14n#") && !uri.equals("http://www.w3.org/2001/10/xml-exc-c14n#WithComments") && !uri.equals("http://www.w3.org/TR/2001/REC-xml-c14n-20010315") && !uri.equals("http://www.w3.org/TR/2001/REC-xml-c14n-20010315#WithComments") && !uri.equals("http://www.w3.org/2006/12/xml-c14n11")) {
                        if (uri.equals("http://www.w3.org/2006/12/xml-c14n11#WithComments")) {
                            break;
                        }
                        xmlSignatureInput = item.performTransform(xmlSignatureInput, null);
                        ++i;
                    }
                    else {
                        break;
                    }
                }
                xmlSignatureInput.setSourceURI(xmlSignatureInput.getSourceURI());
                return xmlSignatureInput;
            }
            return xmlSignatureInput;
        }
        catch (IOException ex) {
            throw new XMLSignatureException(ex);
        }
        catch (ResourceResolverException ex2) {
            throw new XMLSignatureException(ex2);
        }
        catch (CanonicalizationException ex3) {
            throw new XMLSignatureException(ex3);
        }
        catch (InvalidCanonicalizerException ex4) {
            throw new XMLSignatureException(ex4);
        }
        catch (TransformationException ex5) {
            throw new XMLSignatureException(ex5);
        }
        catch (XMLSecurityException ex6) {
            throw new XMLSignatureException(ex6);
        }
    }
    
    public String getHTMLRepresentation() throws XMLSignatureException {
        try {
            XMLSignatureInput nodesetBeforeFirstCanonicalization = this.getNodesetBeforeFirstCanonicalization();
            Transforms transforms = this.getTransforms();
            Transform transform = null;
            if (transforms != null) {
                int i = 0;
                while (i < transforms.getLength()) {
                    Transform item = transforms.item(i);
                    String uri = item.getURI();
                    if (uri.equals("http://www.w3.org/2001/10/xml-exc-c14n#") || uri.equals("http://www.w3.org/2001/10/xml-exc-c14n#WithComments")) {
                        transform = item;
                        break;
                    }
                    ++i;
                }
            }
            HashSet prefixStr2Set = new HashSet();
            if (transform != null && transform.length("http://www.w3.org/2001/10/xml-exc-c14n#", "InclusiveNamespaces") == 1) {
                prefixStr2Set = InclusiveNamespaces.prefixStr2Set(new InclusiveNamespaces(XMLUtils.selectNode(transform.getElement().getFirstChild(), "http://www.w3.org/2001/10/xml-exc-c14n#", "InclusiveNamespaces", 0), this.getBaseURI()).getInclusiveNamespaces());
            }
            return nodesetBeforeFirstCanonicalization.getHTMLRepresentation(prefixStr2Set);
        }
        catch (TransformationException ex) {
            throw new XMLSignatureException(ex);
        }
        catch (InvalidTransformException ex2) {
            throw new XMLSignatureException(ex2);
        }
        catch (XMLSecurityException ex3) {
            throw new XMLSignatureException(ex3);
        }
    }
    
    public XMLSignatureInput getTransformsOutput() {
        return this.transformsOutput;
    }
    
    public ReferenceData getReferenceData() {
        return this.referenceData;
    }
    
    protected XMLSignatureInput dereferenceURIandPerformTransforms(OutputStream outputStream) throws XMLSignatureException {
        try {
            XMLSignatureInput contentsBeforeTransformation = this.getContentsBeforeTransformation();
            this.cacheDereferencedElement(contentsBeforeTransformation);
            return this.transformsOutput = this.getContentsAfterTransformation(contentsBeforeTransformation, outputStream);
        }
        catch (XMLSecurityException ex) {
            throw new ReferenceNotInitializedException(ex);
        }
    }
    
    private void cacheDereferencedElement(XMLSignatureInput xmlSignatureInput) {
        if (xmlSignatureInput.isNodeSet()) {
            try {
                this.referenceData = new ReferenceNodeSetData() {
                    
                    @Override
                    public Iterator<Node> iterator() {
                        return new Iterator<Node>() {
                            Iterator<Node> sIterator = s.iterator();
                            
                            @Override
                            public boolean hasNext() {
                                return this.sIterator.hasNext();
                            }
                            
                            @Override
                            public Node next() {
                                return (Node)this.sIterator.next();
                            }
                            
                            @Override
                            public void remove() {
                                throw new UnsupportedOperationException();
                            }
                        };
                    }
                };
            }
            catch (Exception obj2) {
                Reference.LOG.warn("cannot cache dereferenced data: " + obj2);
            }
        }
        else if (xmlSignatureInput.isElement()) {
            this.referenceData = new ReferenceSubTreeData(xmlSignatureInput.getSubNode(), xmlSignatureInput.isExcludeComments());
        }
        else {
            if (!xmlSignatureInput.isOctetStream() && !xmlSignatureInput.isByteArray()) {
                return;
            }
            try {
                this.referenceData = new ReferenceOctetStreamData(xmlSignatureInput.getOctetStream(), xmlSignatureInput.getSourceURI(), xmlSignatureInput.getMIMEType());
            }
            catch (IOException obj) {
                Reference.LOG.warn("cannot cache dereferenced data: " + obj);
            }
        }
    }
    
    public Transforms getTransforms() throws XMLSignatureException, InvalidTransformException, TransformationException, XMLSecurityException {
        return this.transforms;
    }
    
    public byte[] getReferencedBytes() throws ReferenceNotInitializedException, XMLSignatureException {
        try {
            return this.dereferenceURIandPerformTransforms(null).getBytes();
        }
        catch (IOException ex) {
            throw new ReferenceNotInitializedException(ex);
        }
        catch (CanonicalizationException ex2) {
            throw new ReferenceNotInitializedException(ex2);
        }
    }
    
    private byte[] calculateDigest(boolean b) throws ReferenceNotInitializedException, XMLSignatureException {
        XMLSignatureInput contentsBeforeTransformation = this.getContentsBeforeTransformation();
        if (contentsBeforeTransformation.isPreCalculatedDigest()) {
            return this.getPreCalculatedDigest(contentsBeforeTransformation);
        }
        this.cacheDereferencedElement(contentsBeforeTransformation);
        MessageDigestAlgorithm messageDigestAlgorithm = this.getMessageDigestAlgorithm();
        messageDigestAlgorithm.reset();
        try (final DigesterOutputStream out = new DigesterOutputStream(messageDigestAlgorithm);
             final UnsyncBufferedOutputStream unsyncBufferedOutputStream = new UnsyncBufferedOutputStream(out)) {
            XMLSignatureInput contentsAfterTransformation = this.getContentsAfterTransformation(contentsBeforeTransformation, unsyncBufferedOutputStream);
            this.transformsOutput = contentsAfterTransformation;
            if (Reference.useC14N11 && !b && !contentsAfterTransformation.isOutputStreamSet() && !contentsAfterTransformation.isOctetStream()) {
                if (this.transforms == null) {
                    (this.transforms = new Transforms(this.getDocument())).setSecureValidation(this.secureValidation);
                    this.getElement().insertBefore(this.transforms.getElement(), this.digestMethodElem);
                }
                this.transforms.addTransform("http://www.w3.org/2006/12/xml-c14n11");
                contentsAfterTransformation.updateOutputStream(unsyncBufferedOutputStream, true);
            }
            else {
                contentsAfterTransformation.updateOutputStream(unsyncBufferedOutputStream);
            }
            unsyncBufferedOutputStream.flush();
            if (contentsAfterTransformation.getOctetStreamReal() != null) {
                contentsAfterTransformation.getOctetStreamReal().close();
            }
            return out.getDigestValue();
        }
        catch (XMLSecurityException ex) {
            throw new ReferenceNotInitializedException(ex);
        }
        catch (IOException ex2) {
            throw new ReferenceNotInitializedException(ex2);
        }
    }
    
    private byte[] getPreCalculatedDigest(XMLSignatureInput xmlSignatureInput) throws ReferenceNotInitializedException {
        Reference.LOG.debug("Verifying element with pre-calculated digest");
        return XMLUtils.decode(xmlSignatureInput.getPreCalculatedDigest());
    }
    
    public byte[] getDigestValue() throws XMLSecurityException {
        if (this.digestValueElement == null) {
            throw new XMLSecurityException("signature.Verification.NoSignatureElement", new Object[] { "DigestValue", "http://www.w3.org/2000/09/xmldsig#" });
        }
        return XMLUtils.decode(XMLUtils.getFullTextChildrenFromNode(this.digestValueElement));
    }
    
    public boolean verify() throws ReferenceNotInitializedException, XMLSecurityException {
        byte[] digestValue = this.getDigestValue();
        byte[] calculateDigest = this.calculateDigest(true);
        int equal = MessageDigestAlgorithm.isEqual(digestValue, calculateDigest);
        if (equal == 0) {
            Reference.LOG.warn("Verification failed for URI \"" + this.getURI() + "\"");
            Reference.LOG.warn("Expected Digest: " + XMLUtils.encodeToString(digestValue));
            Reference.LOG.warn("Actual Digest: " + XMLUtils.encodeToString(calculateDigest));
        }
        else {
            Reference.LOG.debug("Verification successful for URI \"{}\"", new Object[] { this.getURI() });
        }
        return equal;
    }
    
    @Override
    public String getBaseLocalName() {
        return "Reference";
    }
    
    static {
        Reference.useC14N11 = (boolean)AccessController.doPrivileged(()/*  */ -> Boolean.valueOf(Boolean.getBoolean("com.sun.org.apache.xml.internal.security.useC14N11")));
        LOG = LoggerFactory.getLogger(Reference.class);
    }
}
